@@ -1,7 +1,7 @@
 # copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: canvaswm.tcl,v 1.7 1998/12/26 10:43:44 jfontain Exp $}
+set rcsId {$Id: canvaswm.tcl,v 1.8 1999/01/29 21:53:06 jfontain Exp $}
 
 class canvasWindowManager {
 
@@ -28,7 +28,7 @@ class canvasWindowManager {
         variable ${this}data
 
         delete [set ${this}data(handle,$path)]
-        unset ${this}data(handle,$path)
+        unset ${this}data(handle,$path) ${this}data(relativeStackingLevel,$path)
     }
 
     proc configure {this path args} {
