@@ -1,11 +1,11 @@
 # copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: canvhand.tcl,v 1.27 1999/03/06 17:42:45 jfontain Exp $}
+set rcsId {$Id: canvhand.tcl,v 1.29 1999/07/17 19:46:46 jfontain Exp $}
 
 class canvasWindowManager {
 
-    class handles {     ;# embed widget in a frame with handles for resizing and moving inside a canvas acting as a window manager
+    class handles {       ;# embed widget in a frame with handles for resizing and moving inside a canvas acting as a window manager
 
         proc handles {this parentPath manager args} composite {[new frame $parentPath] $args} {
             if {[string compare [winfo class $parentPath] Canvas]!=0} {
@@ -14,6 +14,7 @@ class canvasWindowManager {
             set ($this,item) [$parentPath create window 0 0 -window $widget::($this,path) -anchor nw]
             set ($this,manager) $manager
             set ($this,canvas) $parentPath
+            set ($this,filled) 0
             composite::complete $this
         }
 
@@ -36,6 +37,7 @@ class canvasWindowManager {
                 [list -setx setX SetX 0 0]\
                 [list -sety setY SetY 0 0]\
                 [list -static static Static 0]\
+                [list -title title Title {} {}]\
             ]
         }
 
@@ -44,13 +46,16 @@ class canvasWindowManager {
         }
 
         proc set-path {this value} {  ;### mandatory construction time option: eventually enforce or make a constructor argument ###
+            if {[set ($this,filled)]} {
+                error {cannot manage more than 1 widget}
+            }
             if {![winfo exists $value]} {
                 error "invalid widget: \"$value\""
             }
             set path $widget::($this,path)
-            catch {eval pack forget [pack slaves $path]}                                        ;# eventually forget existing widget
-            pack $value -in $path -fill both -expand 1                                                    ;# expand as manager frame
+            pack $value -in $path -side bottom -fill both -expand 1                                       ;# expand as manager frame
             stack $this raise                                            ;# newly managed widgets always appear on top of the others
+            set ($this,filled) 1
         }
 
         foreach option {-background -relief -borderwidth} {
@@ -74,8 +79,26 @@ class canvasWindowManager {
         }
 
         proc set-static {this value} {
+            updateBindings $this $value
+        }
+
+        proc set-title {this value} {
+            if {![info exists ($this,label)]} {
+                set ($this,label) [label $widget::($this,path).label -pady 0 -font $font::(smallNormal) -background gray -anchor w]
+                pack [set ($this,label)] -side top -fill x -before $composite::($this,-path)
+                if {[string length $composite::($this,-path)>0]} {
+                    # always place before the displayed widget so that when the user shrinks the window too much,
+                    # title area always remains visible
+                    pack [set ($this,label)] -before $composite::($this,-path)
+                }
+            }
+            [set ($this,label)] configure -text $value
+            updateBindings $this $composite::($this,-static)
+        }
+
+        proc updateBindings {this static} {
             set path $widget::($this,path)
-            if {$value} {
+            if {$static} {
                 bind $path <Configure> {}
                 bind $path <Motion> {}
                 bind $path <Enter> {}
@@ -92,6 +115,20 @@ class canvasWindowManager {
                 bind $path <ButtonPress-1> "canvasWindowManager::handles::buttonPress $this %x %y"
                 bind $path <ButtonRelease-1> "canvasWindowManager::handles::buttonRelease $this"
             }
+            if {[info exists ($this,label)]} {                                                                         ;# title area
+                set path [set ($this,label)]
+                if {$static} {
+                    $path configure -cursor arrow
+                    bind $path <Button1-Motion> {}
+                    bind $path <ButtonPress-1> {}
+                    bind $path <ButtonRelease-1> "canvasWindowManager::handles::toggleVisibility $this"
+                } else {                                                                      ;# allow moving window from title area
+                    $path configure -cursor fleur
+                    bind $path <Button1-Motion> "canvasWindowManager::handles::buttonMotion $this %x %y"
+                    bind $path <ButtonPress-1> "canvasWindowManager::handles::buttonPress $this %x %y"
+                    bind $path <ButtonRelease-1> "canvasWindowManager::handles::buttonRelease $this"
+                }
+            }
         }
 
         proc buttonMotion {this x y} {
