@@ -1,7 +1,7 @@
 # copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: datatab.tcl,v 1.56 1999/03/06 17:42:45 jfontain Exp $}
+set rcsId {$Id: datatab.tcl,v 1.58 1999/05/01 19:05:35 jfontain Exp $}
 
 class dataTable {                       ;# module data view in the form of a table, that can be sorted using a column as a reference
 
@@ -13,7 +13,7 @@ class dataTable {                       ;# module data view in the form of a tab
     proc dataTable {this parentPath args} composite {
         [new scroll table $parentPath\
             -scrollbarwidth $dataTable::(scrollbarWidth) -scrollbarelementborderwidth $dataTable::(scrollbarBorderWidth)\
-            -width $configuration::(default,viewer,width) -height $configuration::(default,viewer,height)\
+            -width $global::viewerWidth -height $global::viewerHeight\
         ]
         $args
     } {
@@ -25,7 +25,8 @@ class dataTable {                       ;# module data view in the form of a tab
             -takefocus 0
         $path tag configure select -background white
 
-        bindtags $path "$path . all"           ;# remove all class bindings for we do not use any and they would cause interferences
+        # remove all class bindings for we do not use any and they would cause interferences
+        bindtags $path [list $path [winfo toplevel $path] all]
 
         # allow border resizing with first button. does not interfere with drag bindings since command does nothing unless mouse
         # click occured in a column border, which cannot be the case when dragging (see drag validation procedure in this class)
