@@ -72,7 +72,7 @@ case of processes running on a computer system, rows might be sorted according
 to their unique process identifier, with columns containing values such
 as CPU usage, memory usage, owner's name, time of creation, ...
 <p>The software used to view this type of information comes in different
-forms and shapes. Unix users might be familiar with the "top" application
+forms and shapes. Unix users might be familiar with the <i>top</i> application
 which presents rows of process data as lines of text, whereas RMON (Remote
 MONitoring) SNMP software usually uses multiple windows with graphical
 displays, curves, pie charts, multiple configuration dialog boxes, even
@@ -85,44 +85,43 @@ could be used. The user could then sort table rows, select one or more
 cells, rows, columns, create views such as other tables, charts, ... best
 suited to the way data should be presented. Once optimized, the data viewers
 layout and configuration could be saved for later reuse as a dashboard.
-In effect, what is needed is a spreadsheet that is capable of dealing with
-dynamically changing data.
+In effect, what is needed is a spreadsheet tailored to dynamic data processing.
 <p>Moodss (Modular Object Oriented Dynamic SpreadSheet) is an attempt at
 answering these needs. It is composed of a main part (the core) and an
 unlimited number of modules, loaded as the application is launched, each
 module interfacing to a specific type of data. The core is written in the
 great Tcl language (at <a href="http://www.scriptics.com/">http://www.scriptics.com/</a>)
-using object oriented techniques thanks to the stooop and scwoop packages
-(at <a href="http://www.multimania.com/jfontain/">http://www.multimania.com/jfontain/</a>).
+using object oriented techniques thanks to the stooop package (at <a href="http://www.multimania.com/jfontain/">http://www.multimania.com/jfontain/</a>).
 The module function is to describe the data that it is also in charge of
 retrieving and formatting. Modules can be written in plain Tcl or use dynamically
 linked libraries written in the C language (modules are packages in the
 Tcl sense, so any language that can interface with Tcl is supported).
 <p>Modules are loaded when moodss is started. Several modules can be handled
 concurrently (starting with moodss version 3.0). This way, you may monitor
-data coming from any number of heterogeneous sources. Module names are
-specified in the command line and cannot be dynamically unloaded.
+data coming from any number of heterogeneous sources. Modules are specified
+in the command line and cannot be dynamically unloaded.
 <p>Versions 4.0 and up add a dashboard functionality: the current configuration
 (modules, viewers, poll time, windows sizes and placement, ...) can be
 saved in a file at any time, for later reuse (see the -f (--file) command
 line switch documentation).
-<p>Versions 4.3 and up support asynchronous modules (no polling needed
-as module data may change on its own). Note that any number of asynchronous
-and regular (synchronous) modules can be simultaneously loaded.
+<p>Versions 4.3 and up support asynchronous modules (for which no polling
+is needed as module data may change on its own). Note that any number of
+asynchronous and regular (synchronous) modules can be simultaneously loaded.
 <p>Versions 5.0 and up add a free text viewer, which can be used for comments,
 and which can also embed live data cells in text form.
 <p>Versions 5.3 and up support viewer type mutation through viewer icon
-drag'n'drop, and viewer quick destruction through trash icon drag'n'drop
-operation. A new menu for empty viewer creation was also added.
+drag'n'drop, and viewer quick destruction by a drop into the trash icon.
+A new menu for empty viewer creation was also added.
 <p>Versions 6.0 and up support command line arguments per module, data
 table column anchoring in module configuration and automatic module discovery.
-<p>Versions 6.1 and up support HTML formatted data for modules.
+<p>Versions 6.1 and up support HTML formatted help data for modules.
 <p>Versions 6.6 and up support automatic cross hairs with coordinates in
 message area for graph data viewers.
 <p>Versions 6.7 and up support contextual help (through the main window
 message area) on all menu items.
 <p>Versions 7.0 and up support per user (on UNIX systems) application wide
 preferences setting.
+<p>Versions 7.99 and up support concurrent instances of the same module.
 <p>Since module data access is entirely customizable (through C code, Tcl,
 HTTP, ...) and since several modules can be loaded at once, applications
 for moodss become limitless. For example, comparing a remote database server
@@ -150,9 +149,11 @@ menu for more information.
 If you are using a Linux Redhat system (5.1 or above), then use the moodss
 rpm file (available (starting with 4.4) at <a href="http://www.multimania.com/jfontain/">http://www.multimania.com/jfontain/</a>)
 for installation. It requires the tcl, tk, blt and tktable rpms (see INSTALL
-file for more information). Unless you want to work on the moodss source
-code, you can skip the rest of this section.
-<p>For the current version (7.99), the following packages must be installed
+file for more information). You are then all set for using the included
+Linux modules or develop your own modules.
+<p><b><i>Unless you want to work on the moodss source code (not the modules),
+you can skip the rest of this section.</i></b>
+<p>For the current version (8.0), the following packages must be installed
 before attempting to install moodss (make sure to check the INSTALL file
 for the latest information):
 <ul>
@@ -194,13 +195,13 @@ The moodss application is composed of the core software and one or several
 modules. Modules are implemented as Tcl packages and thus usually comprise
 a Tcl source file and a pkgIndex.tcl file as required by the Tcl package
 implementation.
-<p>The core will load one or more modules, whose names were passed as command
-line parameters or come from a save file, and will start displaying module
+<p>The core loads one or more modules, whose names are passed as command
+line parameters or come from a save file, and starts displaying module
 data in one or more tables. The tables are then updated at the frequency
 defined by the poll time, which the user may change, or asynchronously
 for the relevant modules. For example, to launch moodss with the random
-module, just type:
-<pre>$ wish moodss random</pre>
+module, just type (on a UNIX machine):
+<pre>$ moodss random</pre>
 All the module code and data are kept in a separate namespace. The module
 data is stored is a single array including some configuration data used
 when the module is loaded by the core, and variable data (displayed in
@@ -209,17 +210,17 @@ it must start updating its data when requested by the core. If a module
 is asynchronous, its data may be updated at any time. The synchronous or
 asynchronous nature is specified in the configuration data for the module.
 <p>The initial data tables represent the first data views, from which any
-number of cells can be selected. Data viewers can be created by dropping
-cells through a drag'n'drop operation into a graph, bar chart, pie chart,
-summary table or free text iconic site. In turn, these viewers can display
-more table cells, which when dropped into the viewer, result in the creation
-of corresponding data graph lines, chart bars, pie slices, table rows or
-text cells. Cells or rows can be removed from existing viewers, by simply
-selecting them and dropping them in the trash iconic site (a bullet hole).
-<p>Any viewer can be mutated (its type changed) by dropping a viewer icon
-into it. For example, create a stacked data bar viewer from several cells,
-then drag the 3D pie drag'n'drop icon into it. Any viewer can also be destroyed
-in one shot by dropping the trash icon into it.
+number of cells can be selected. Data viewers can be created by dragging
+and dropping cells into a graph, bar chart, pie chart, summary table or
+free text iconic site. In turn, these viewers can display more table cells,
+which when dropped into the viewer, result in the creation of corresponding
+data graph lines, chart bars, pie slices, table rows or text cells. Cells
+or rows can be removed from existing viewers, by simply selecting them
+and dropping them in the trash iconic site (a bullet hole).
+<p>Any viewer can be mutated (its type changed) by by dragging from a viewer
+icon and dropping into it. For example, create a stacked data bar viewer
+from several cells, then drag the 3D pie icon into it. Any viewer can also
+be destroyed in one shot by dropping the trash icon into it.
 <p>Any draggable data can be dropped in any valid drop site at any time.
 It is thus possible to drag several data cells from any table or any viewer
 into other ones, the trash, ... even if the data comes from different modules.
@@ -237,10 +238,10 @@ Immediately after launch, module(s) is(are) loaded and initialized, with
 corresponding messages displayed in the message area, as follows:
 <center><pre><img SRC="moodss5.gif" height=54 width=234></pre></center>
 Soon after, tabular data is displayed in one or more tkTable widgets with
-automatic scroll bars, between the menu bar, the drop sites with graphical
-viewers, summary table, free text and trash icons and a message area, as
-one can see below:
-<center><pre><img SRC="moodss1.gif" ALT="moodss initial main window view" NOSAVE height=501 width=501></pre></center>
+the module identifier as title, automatic scroll bars, between the menu
+bar, the drop sites with graphical viewers, summary table, free text and
+trash icons and a message area, as one can see below:
+<center><pre><img SRC="moodss1.gif" ALT="moodss initial main window view" NOSAVE height=515 width=505></pre></center>
 The message area is used to display status information, such as when the
 data is being updated, and help information, as the user moves the mouse
 pointer over sensitive areas, such as table column headers. Contextual
@@ -251,6 +252,11 @@ Further help is provided through widget tips (also known as balloons) when
 appropriate, and of course the Help menu.
 <p>The window title shows the name of the loaded module(s) along with the
 poll time.
+<p>When several modules of the same type are loaded (for example, CPU statistics
+on a group of servers), the initial data tables feature the module name
+followed by an instance number (<i>module&lt;N></i>), or the module identifier
+generated from the module code (<i>cpustats(host.domain.org)</i> for example).
+A lone module keeps his unmodified name as table titles.
 <p>The <i>File</i> menu contains the <i>Save</i>, <i>Save As</i> and <i>Exit</i>
 menu entries, used to save the current configuration (modules and viewers),
 or gracefully quit the moodss application.
@@ -259,7 +265,7 @@ when not running in read-only mode, see <a href="#commandline">Command
 line</a>), which allows the creation of empty viewers of any type, and
 the <i>Preferences</i> entry which when selected launches the following
 dialog box:
-<center><pre><img SRC="moodss6.gif" height=195 width=482></pre></center>
+<center><pre><img SRC="moodss6.gif" height=200 width=446></pre></center>
 The Preferences dialog box allows the user to change application wide settings,
 saved in a global file (known as an <i>rc</i> file to UNIX people), used
 for initialization when the application is started. Since this file is
@@ -273,8 +279,7 @@ as shown below:
 <center><pre><img SRC="moodss2.gif" ALT="poll time dialog box" NOSAVE height=140 width=161></pre></center>
 The user can select a new poll time among the module choices from a spin
 entry widget, or directly type in a new value, as long as it is not smaller
-than the module minimum poll time, in which case a message dialog box warns
-the user.
+than the module minimum poll time, in which case a warning message is displayed.
 <p>When several modules are used, the minimum poll time is the greater
 of the minimum poll times of all modules. The default poll time (used when
 moodss is started) is the greater of the default poll times of all modules.
@@ -289,23 +294,28 @@ in opposite order, thus toggling between increasing and decreasing orders.
 <br>When sorting, the selected column is used as a reference, meaning that
 all rows will be rearranged so that the selected column appears sorted,
 with values either increasing or decreasing.
-<br>A little arrow indicator is placed to the right of the reference column
-title label, pointing up or down depending on whether the sorting order
-is decreasing or increasing.
+<br>A little triangular indicator is placed to the right of the reference
+column title label, pointing up or down depending on whether the sorting
+order is decreasing or increasing.
 <br>Table columns can be interactively resized by holding the first mouse
 button down on a column border. The mouse cursor is changed to an horizontal
 double arrow on column borders to show this capability.
 <p>Aside from the main tables, graphical and textual viewers can be created
-for viewing table cell data behavior over time. Viewers can also be deleted,
+for monitoring table cell data over time. Viewers can also be deleted,
 data views (such as pie slices, curves, ...) can be added or removed from
 existing viewers, ... These functions are all implemented using the drag
 and drop functionality described below.
-<p>Graphical viewers available at this time are BLT graph viewers (such
-as can be seen below), side-by-side bars charts, overlapped bars charts,
-stacked bars charts, 2D pie charts and 3D pie charts*. Graph viewers feature
-an automatic cross hair which follows the mouse pointer movements inside
-the plotting area. Corresponding coordinates are updated in real time in
-the main window message area.
+<p>For all viewers, if a module identifier string is required (provided
+by the module, several instances of the same module, ...) for proper cell
+identification, that string will be placed first in the label. For example,
+data cells originating from the third instance of the random module would
+be labeled: "<i>random&lt;3>: data cell label"</i>.
+<p>Graphical viewers available at this time are BLT graph viewers (see
+images below), side-by-side bars charts, overlapped bars charts, stacked
+bars charts, 2D pie charts and 3D pie charts*. Graph viewers feature an
+automatic cross hair which follows the mouse pointer movements inside the
+plotting area. Corresponding coordinates are updated in real time in the
+main window message area.
 <p>*<i>note: if you know of any other nice viewers (like 3D graphs) that
 work with Tcl, please let me know so I can integrate them. Many thanks
 in advance...</i>
@@ -345,11 +355,13 @@ There are 2 textual viewers.
 average, minimum and maximum values since the row was created. Data cells
 can be inserted one or several at a time through a simple drop. Rows can
 be deleted by selecting any cell in the row then dropping any number of
-them into the trash drop site.
-<p>The free text viewer is an editable Tcl text widget with any number
-(including zero) of embedded data cell windows. Data cells can be inserted
-one or several at a time through a simple drop, as with the other viewers.
-New data cell windows are inserted at the current insertion cursor position.
+them into the trash drop site. Data cells with missing data (could be no
+longer available if coming from a vanished process, for example) display
+the 000000 pattern.
+<p>The free text viewer is an editable Tk text widget with any number (including
+zero) of embedded data cell windows. Data cells can be inserted one or
+several at a time through a simple drop, as with the other viewers. New
+data cell windows are inserted at the current insertion cursor position.
 Data cells can be deleted by selecting then dropping any number of them
 into the trash drop site. They can also be deleted using the keyboard Delete
 and Backspace keys, which also work on the regular text, as well as the
@@ -369,7 +381,7 @@ at any time.
 </table></center>
 Here is a screen shot of loaded <i>ps</i> and <i>cpustats</i> modules with
 several graphical viewers:
-<center><pre><img SRC="moodss3.gif" ALT="moodss window with graph data viewers" NOSAVE height=514 width=524></pre></center>
+<center><pre><img SRC="moodss3.gif" ALT="moodss window with graph data viewers" NOSAVE height=553 width=538></pre></center>
 All data viewers can be moved and resized thanks to handling areas in the
 data viewer borders. When moving the mouse pointer over these areas, the
 mouse cursor changes to indicate the possible action. Corner handles allow
@@ -385,13 +397,14 @@ small window manager functionality is useless, as it behaves like a basic
 window manager (let me know if it does not).
 <p>Here is another shot featuring a free text viewer with loaded <i>cpustats</i>
 and <i>memstats</i> modules:
-<center><pre><img SRC="moodss4.gif" ALT="moodss window with free text data viewer" NOSAVE height=480 width=459></pre></center>
+<center><pre><img SRC="moodss4.gif" ALT="moodss window with free text data viewer" NOSAVE height=509 width=493></pre></center>
 The <i>Help</i> menu contains <i>Global</i> help (actually launches an
-embedded HTML viewer with this very document), <i>Modules</i> menu with
-one sub menu per module (which displays the module's help data), <i>Copyright</i>
-which displays the GPL (Gnu General Public License) document, <i>Source
-Versions </i>which display all the source code file names with their versions
-and <i>About</i> general information entries.
+embedded HTML viewer with this very document minus the sections related
+to module development), <i>Modules</i> menu with one sub menu per module
+(which displays the module's help data), <i>Copyright</i> which displays
+the GPL (Gnu General Public License) document, <i>Source Versions </i>which
+display all the source code file names with their versions and <i>About</i>
+general information entries.
 <h5>
 <a NAME="draganddrop"></a>5.1.1 Drag and drop</h5>
 Drag and drop in moodss tries to behave as the now familiar Windows functionality
@@ -548,7 +561,15 @@ as parameters, as in:
 <pre>&nbsp;&nbsp;&nbsp; $ wish moodss random</pre>
 or, for 2 modules at once:
 <pre>&nbsp;&nbsp;&nbsp; $ wish moodss ps cpustats</pre>
-You may not specify the same module more than once in the command line.
+You can specify the same module more than once and with different arguments
+in the command line:
+<pre>&nbsp;&nbsp;&nbsp; $ wish moodss ps -r host.domain ps -r otherhost.domain</pre>
+When several modules of the same type are passed as argument, the initial
+data tables feature the module name followed by a number as title. For
+example <i>"ps&lt;2>"</i>. If the module provides an identifier string,
+that text will be used instead, as in <i>"ps(host.domain)"</i>. Data cell
+labels include the module identifier or numbered module name as well, for
+proper identification.
 <p>You may eventually specify a poll time in seconds using:
 <pre>&nbsp;&nbsp;&nbsp; $ wish moodss -p 25 random</pre>
 Note that when all the specified modules are asynchronous, the poll time
