@@ -1,11 +1,11 @@
 # copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: memstats.tcl,v 1.9 1999/02/26 21:11:32 jfontain Exp $}
+set rcsId {$Id: memstats.tcl,v 1.11 1999/07/23 21:26:40 jfontain Exp $}
 
 
-package provide memstats 1.3
-package require network 1.2
+package provide memstats 1.4
+package require network 1.3
 
 namespace eval memstats {
 
@@ -25,7 +25,7 @@ This is a view of memory and swap statistics for the system as a whole.
 
 Module options:
 
--r (--remote) [user@]host: remote monitoring using "user" as logname on remote host "host" (rsh facilities must be properly setup). If user is not specified, current user is used as logname on remote host.
+-r (--remote) [user@]host: remote monitoring using "user" as logname on remote host "host" (rsh facilities must be properly setup). If user is not specified, current user is used as logname on remote host. The module title is set to "memstats(host)".
 
 Examples:
 
@@ -45,6 +45,8 @@ $ moodss ps --remote foo.bar.com
         if {![catch {set locator $options(--remote)}]||![catch {set locator $options(-r)}]} {                   ;# remote monitoring
             set data(pollTimes) {20 10 30 60 120 300 600}                                ;# poll less often when remotely monitoring
             foreach {remote(user) remote(host)} [network::parseRemoteLocator $locator] {}
+            network::checkRemoteOutputEmptiness $remote(host)
+            set data(identifier) memstats($remote(host))
             set file [open "| /usr/bin/rsh -nl $remote(user) $remote(host) cat /proc/meminfo"]
             fileevent $file readable {set ::memstats::remote(busy) 0}
             vwait ::memstats::remote(busy)                                                                  ;# allow GUI interaction
