@@ -1,13 +1,12 @@
 # copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: modules.tcl,v 1.12 1999/07/14 15:48:46 jfontain Exp $}
+set rcsId {$Id: modules.tcl,v 1.18 1999/07/23 21:16:28 jfontain Exp $}
 
 class modules {
 
     set modules::(names) {}
-    set modules::(indexed) {}
-    set modules::(duplicates) 0                            ;# boolean to indicate whether there exists several instances of a module
+    set modules::(namespaces) {}
 
     proc modules {this} error                                                                                   ;# object-less class
 
@@ -34,31 +33,33 @@ class modules {
     proc parse {arguments} {         ;# arguments list format is: module [-option [value] -option ...] module [-option ...]
         if {[llength $arguments]==0} return                                                                         ;# nothing to do
         set module [lindex $arguments 0]
+
         set arguments [lrange $arguments 1 end]                                         ;# point to start of switches or next module
 
-        scan $module {%[^<]<%u>} module index    ;# eventually split module into its name and its index (if coming from a save file)
+        # eventually split module into its name and its index (if coming from a save file)
+        foreach {module index} [decoded $module] {}
 
         if {![info exists ::packageDirectory($module)]} {                             ;# not a valid module (usually a wrong switch)
             puts stderr "error: \"$module\" is not a valid moodss module name"
             exit 1
         }
+        if {![validName $module]} {
+            puts stderr "\"$module\" module name contains invalid characters"
+            exit
+        }
 
         lifoLabel::push $global::messenger "loading $module..."
         update idletasks
 
-        if {[info exists index]} {                                                                             ;# force module index
-            set indexed [load $module $index]
-        } else {
-            set indexed [load $module]
-        }
+        set namespace [load $module $index]
 
         lifoLabel::pop $global::messenger
-        lappend modules::(indexed) $indexed                      ;# we never get here if there is an error when the module is loaded
+        lappend modules::(namespaces) $namespace                 ;# we never get here if there is an error when the module is loaded
         if {[lsearch -exact $modules::(names) $module]<0} {                                     ;# keep track of loaded module names
             lappend modules::(names) $module
         }
-        if {[catch {set ::${indexed}::data(switches)} switches]} {                                        ;# module takes no options
-            set modules::($indexed,arguments) {}                                ;# save module arguments for eventual saving in file
+        if {[catch {set ::${namespace}::data(switches)} switches]} {                                      ;# module takes no options
+            set modules::($namespace,arguments) {}                              ;# save module arguments for eventual saving in file
         } else {                                                                                             ;# module takes options
             if {[llength $switches]==0} {
                 puts stderr "module \"$module\" switches are empty"
@@ -68,26 +69,26 @@ class modules {
                 puts stderr "module \"$module\" options error: $message"
                 exit 1
             }
-            if {!$modules::($indexed,initialize)} {
+            if {!$modules::($namespace,initialize)} {
                 puts stderr "module \"$module\" has no initialize procedure"
                 exit 1
             }
-            set modules::($indexed,options) [array get options]
+            set modules::($namespace,options) [array get options]
             # save module arguments for eventual saving in file
-            set modules::($indexed,arguments) [lrange $arguments 0 [expr {[llength $arguments]-[llength $next]-1}]]
+            set modules::($namespace,arguments) [lrange $arguments 0 [expr {[llength $arguments]-[llength $next]-1}]]
             set arguments $next
         }
         parse $arguments                                                                                         ;# process the rest
         update idletasks                                       ;# make sure latest loading message is not left showing meaninglessly
     }
 
-    proc helpHTMLData {indexed} {  ;# return HTML formatted module help no matter whether provided plain or preformatted from module
-        if {[catch {set ${indexed}::data(helpText)} text]} {
+    proc helpHTMLData {namespace} {       ;# return HTML formatted help no matter whether provided plain or preformatted from module
+        if {[catch {set ${namespace}::data(helpText)} text]} {
             set text {no help available}
         }
-        set header "<H6>module $indexed</H6>\n<I>version $modules::($indexed,version)"
-        if {[string length $modules::($indexed,arguments)]>0} {
-            append header ", invoked with arguments: $modules::($indexed,arguments)"
+        set header "<H6>module $namespace</H6>\n<I>version $modules::($namespace,version)"
+        if {[string length $modules::($namespace,arguments)]>0} {
+            append header ", invoked with arguments: $modules::($namespace,arguments)"
         }
         append header </I><BR><BR>\n
         if {[string first <BODY> $text]<0} {                                                                         ;# regular help
@@ -100,16 +101,23 @@ class modules {
     }
 
     proc initialize {} {          ;# eventually invoke modules initialization procedures. modules must be loaded first (see parse{})
-        foreach indexed $modules::(indexed) {
-            if {!$modules::($indexed,initialize)} continue
-            lifoLabel::push $global::messenger "initializing $indexed..."
+        foreach namespace $modules::(namespaces) {
+            if {!$modules::($namespace,initialize)} continue
+            lifoLabel::push $global::messenger "initializing $namespace..."
             update idletasks
-            if {[info exists modules::($indexed,options)]} {
-                ::${indexed}::initialize $modules::($indexed,options)                                  ;# let module initialize self
+            if {[info exists modules::($namespace,options)]} {
+                ::${namespace}::initialize $modules::($namespace,options)                              ;# let module initialize self
             } else {                                                                                      ;# module takes no options
-                ::${indexed}::initialize                                                               ;# let module initialize self
+                ::${namespace}::initialize                                                             ;# let module initialize self
+            }
+            synchronize $namespace                                                                       ;# in case data was updated
+            if {![catch {set ${namespace}::data(identifier)} identifier]} {            ;# store identifier if it exists and is valid
+                if {![validName $identifier]} {
+                    puts stderr "\"$namespace\" module identifier: \"$identifier\" contains invalid characters"
+                    exit
+                }
+                set modules::($namespace,identifier) $identifier
             }
-            synchronize $indexed                                                                         ;# in case data was updated
             lifoLabel::pop $global::messenger
         }
         update idletasks                                ;# make sure latest initialization message is not left showing meaninglessly
@@ -118,10 +126,10 @@ class modules {
     proc setPollTimes {{override {}}} {
         set default 0
         set minimum 0
-        foreach indexed $modules::(indexed) {
-            set times [set ${indexed}::data(pollTimes)]
+        foreach namespace $modules::(namespaces) {
+            set times [set ${namespace}::data(pollTimes)]
             if {[llength $times]==0} {
-                error "module $indexed poll times list is empty"
+                error "module $namespace poll times list is empty"
             }
             # for an asynchronous module, the sole time value would be negative and is used as graph interval, for example
             set time [lindex $times 0]
@@ -136,7 +144,7 @@ class modules {
             set time [lindex $times 0]
             if {$time>$minimum} {
                 set minimum $time                                                    ;# keep the greater minimum time of all modules
-                set minimumModule $indexed
+                set minimumModule $namespace
             }
             foreach time $times {                                    ;# poll times list is the combination of all modules poll times
                 set data($time) {}
@@ -146,7 +154,7 @@ class modules {
         set global::pollTimes [lsort -integer [array names data]]
         set global::pollTimes [lrange $global::pollTimes [lsearch -exact $global::pollTimes $minimum] end]
         set global::pollTime $default
-        if {[string length $override]>0} {                                       ;# eventually validate command line override
+        if {[string length $override]>0} {                                              ;# eventually validate command line override
             if {$override<$minimum} {
                 puts stderr "$::argv0: minimum poll time is $minimum seconds for module $minimumModule"
                 exit 1
@@ -168,37 +176,46 @@ class modules {
     }
 
     # load a module in its own interpreter, in order to allow multiple instances of the same module
-    proc load {module {index 0}} {                                             ;# index may be forced (when coming from a save file)
+    proc load {module index} {            ;# index may be forced (when coming from a save file), left empty for automatic generation
         variable nextIndex
 
-        # maintain an index for each module in case it is loaded several times
-        if {($index==0)&&[catch {set index $nextIndex($module)}]} {
-            set index 1
-        }
-        if {$index>1} {                                                            ;# there are several instances of the same module
-            set modules::(duplicates) 1
+        if {[string length $index]==0} {
+            if {[catch {set index $nextIndex($module)}]} {                                             ;# first instance of a module
+                set namespace $module                                                                    ;# use original module name
+                set index 0
+            } else {
+                set namespace ${module}<$index>                                    ;# this is another instance of an existing module
+            }
+        } else {                                                                                    ;# index was passed as parameter
+            set namespace ${module}<$index>
         }
+        set nextIndex($module) [incr index]
 
-        set indexed ${module}<$index>                                                                          ;# unique module name
-        set modules::($indexed,name) $module
+        set modules::($namespace,module) $module
 
         set interpreter [interp create]                                                ;# use a separate interpreter for each module
-        set modules::($indexed,interpreter) $interpreter
-        set ::packageDirectory($indexed) $::packageDirectory($module)                        ;# set indexed module package directory
+        set modules::($namespace,interpreter) $interpreter
+        set ::packageDirectory($namespace) $::packageDirectory($module)                              ;# set module package directory
+        $interpreter eval {                    ;# since Tk is not loaded in module interpreter, provide a background error procedure
+            proc bgerror {error} {
+                puts stderr $::errorInfo
+                exit 1
+            }
+        }
         $interpreter eval "
             set auto_path [list $::auto_path]                          ;# copy a few required global variables in module interpreter
             set ::packageDirectory($module) $::packageDirectory($module)
             package require $module                                                                         ;# module is loaded here
         "
         # we never get here if there is an error when the module is loaded
-        # the indexed namespace, "interface' to the protected module namespace in its interpreter, is child of global namespace
-        namespace eval ::$indexed [subst -nocommands {
+        # the new namespace, "interface' to the protected module namespace in its interpreter, is child of the global namespace
+        namespace eval ::$namespace [subst -nocommands {
             proc update {} {$interpreter eval ::${module}::update}
         }]
-        set modules::($indexed,initialize)\
+        set modules::($namespace,initialize)\
             [$interpreter eval [subst -nocommands {expr {[string length [namespace eval ::$module {info proc initialize}]]>0}}]]
-        if {$modules::($indexed,initialize)} {                                                        ;# initialize procedure exists
-            namespace eval ::$indexed [subst -nocommands {      ;# create an interface initialize procedure within indexed namespace
+        if {$modules::($namespace,initialize)} {                                                      ;# initialize procedure exists
+            namespace eval ::$namespace [subst -nocommands {        ;# create an interface initialize procedure within new namespace
                 proc initialize {arguments} {                     ;# arguments are a list of option / value (eventually empty) pairs
                     $interpreter eval "
                         array set _options [list \$arguments]
@@ -208,30 +225,55 @@ class modules {
                 }
             }]
         }
-        set modules::($indexed,version) [$interpreter eval "package provide $module"]
-        synchronize $indexed                                        ;# initialize indexed module data from module in its interpreter
+        set modules::($namespace,version) [$interpreter eval "package provide $module"]
+        synchronize $namespace                                           ;# initialize namespace data from module in its interpreter
         # keep on eye on special module data array member "update"
-        $interpreter alias _updated ::modules::updated $indexed
+        $interpreter alias _updated ::modules::updated $namespace
         $interpreter eval "trace variable ::${module}::data(updates) w _updated"
+        # setup interface to messenger:
+        $interpreter alias pushMessage ::lifoLabel::push $::global::messenger
+        $interpreter alias popMessage ::lifoLabel::pop $::global::messenger
+        $interpreter alias flashMessage ::lifoLabel::flash $::global::messenger
 
-        set nextIndex($module) [incr index]
-
-        return $indexed                                                                               ;# new indexed name for module
+        return $namespace                                                                                     ;# new name for module
     }
 
-    proc updated {indexed args} {                              ;# module data was just updated. ignore already known trace arguments
-        synchronize $indexed {[0-9]*,[0-9]*}                                         ;# just copy all dynamic data from module array
+    proc updated {namespace args} {                            ;# module data was just updated. ignore already known trace arguments
+        synchronize $namespace {[0-9]*,[0-9]*}                                       ;# just copy all dynamic data from module array
         # and copy updates counter
-        set ::${indexed}::data(updates) [$modules::($indexed,interpreter) eval "set ::$modules::($indexed,name)::data(updates)"]
+        set ::${namespace}::data(updates)\
+            [$modules::($namespace,interpreter) eval "set ::$modules::($namespace,module)::data(updates)"]
     }
 
-    proc synchronize {indexed {pattern *}} {                      ;# copy data from module in its interpreter to indexed module here
-        array set ::${indexed}::data\
-            [$modules::($indexed,interpreter) eval "array get ::$modules::($indexed,name)::data {$pattern}"]
+    proc synchronize {namespace {pattern *}} {                  ;# copy data from module in its interpreter to module namespace here
+        array set ::${namespace}::data\
+            [$modules::($namespace,interpreter) eval "array get ::$modules::($namespace,module)::data {$pattern}"]
     }
 
-    proc identifier {array} {            ;# from a module data array name, deduce a unique module identifier (used in viewer labels)
-        return [string trimleft [namespace qualifiers [namespace which -variable $array]] :]
+    proc identifier {array} {  ;# from an array name, eventually deduce a unique module identifier if needed (used in viewer labels)
+        variable nextIndex
+
+        set namespace [string trimleft [namespace qualifiers [namespace which -variable $array]] :]
+        if {[lsearch -exact $modules::(namespaces) $namespace]>=0} {                                       ;# this is a module array
+            if {[info exists modules::($namespace,identifier)]} {
+                return $modules::($namespace,identifier)                                            ;# favor identifier if it exists
+            }
+            foreach {module index} [decoded $namespace] {}
+            if {$nextIndex($module)>1} {               ;# there are more than 1 instance of this module, so identification is needed
+                return $namespace
+            }
+        }
+        return {}                                                                   ;# not a module array or identification unneeded
+    }
+
+    proc decoded {module} {   ;# return module and index list (index may be empty if module is not indexed: name instead of name<N>)
+        set index {}
+        scan $module {%[^<]<%u>} module index                                 ;# eventually split module into its name and its index
+        return [list $module $index]
+    }
+
+    proc validName {string} {                                                         ;### also use string is alphanum with 8.1+ ###
+        return [regexp {^[ 0-9a-zA-Z~!@%^&*()_=+|;:',.?-]+$} $string]
     }
 
 }
