@@ -4,7 +4,6 @@ exec wish "$0" "$@"
 
 # copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
-proc echo {args} {foreach argument $args {puts -nonewline "$argument "}; puts {}}
 
 proc recordRCSId {variableName empty operation} {
     foreach {header file version} $::rcsId break
@@ -13,17 +12,9 @@ proc recordRCSId {variableName empty operation} {
 }
 trace variable ::rcsId w recordRCSId                                                                ;# keep track of RCS identifiers
 
-set rcsId {$Id: moodss.tcl,v 1.169 1999/07/14 14:28:13 jfontain Exp $}
+set rcsId {$Id: moodss.tcl,v 1.173 1999/07/21 18:50:24 jfontain Exp $}
 
 
-source utility.tcl
-startGatheringPackageDirectories
-
-source global.tcl
-source preferen.tcl
-preferences::load                                                                     ;# initialize from rc file as soon as possible
-source getopt.tcl
-
 proc printUsage {exitCode} {
     puts stderr "Usage: $::argv0 \[OPTION\]... \[MODULE\] \[MODULE\]..."
     puts stderr {  -f, --file       configuration file name}
@@ -36,11 +27,17 @@ proc printUsage {exitCode} {
     exit $exitCode
 }
 
+source utility.tcl
+startGatheringPackageDirectories
+source getopt.tcl
+
 if {[catch\
     {\
         set argv [parseCommandLineArguments\
-            {-f 1 --file 1 -h 0 --help 0 -p 1 --poll-time 1 -r 0 --read-only 0 -S 0 --static 0 --show-modules 0 --version 0} $argv\
-            arguments\
+            {
+                -f 1 --file 1 -h 0 -he 0 -hel 0 -help 0 --help 0 -p 1 --poll-time 1 -r 0 --read-only 0 -S 0 --static 0
+                --show-modules 0 --version 0
+            } $argv arguments\
         ]\
     }\
     message\
@@ -53,17 +50,25 @@ foreach {short long} {-f --file -h --help -p --poll-time -r --read-only -S --sta
     catch {set arguments($short) $arguments($long)}                                          ;# long version if present has priority
 }
 
-if {[info exists arguments(--version)]} {
-    puts "moodss (a Modular Object Oriented Dynamic SpreadSheet) version $global::applicationVersion"
-    exit
-}
 if {
-    [info exists arguments(-h)]||
+    [info exists arguments(-h)]||[info exists arguments(-he)]||[info exists arguments(-hel)]||[info exists arguments(-help)]||
     (([llength $argv]==0)&&![info exists arguments(-f)]&&![info exists arguments(--show-modules)])
 } {
     printUsage 1
 }
 
+# since it invokes some Tk commands which are not available when -help (or shortened) switch is passed in command line, this file
+# sourcing must appear after -help switch detection and processing
+source global.tcl
+
+if {[info exists arguments(--version)]} {
+    puts "moodss (a Modular Object Oriented Dynamic SpreadSheet) version $global::applicationVersion"
+    exit
+}
+
+source preferen.tcl
+preferences::load                                                                     ;# initialize from rc file as soon as possible
+
 lappend auto_path /usr/lib/moodss .    ;# search in current directory sub-directories for development and rpm installation directory
 
 if {[catch {package require stooop 3.7}]} {
@@ -203,12 +208,12 @@ if {!$readOnly} {
 }
 pack $widget::($scroll,path) -fill both -expand 1
 
-set modules::(synchronous) {}                                                            ;# save list of synchronous indexed modules
+set modules::(synchronous) {}                                                                    ;# save list of synchronous modules
 set names {}
-foreach indexed $modules::(indexed) {
-    if {[lindex [set ${indexed}::data(pollTimes)] 0]>0} {                                                ;# if module is synchronous
-        lappend modules::(synchronous) $indexed
-        set module $modules::($indexed,name)
+foreach namespace $modules::(namespaces) {
+    if {[lindex [set ${namespace}::data(pollTimes)] 0]>0} {                                              ;# if module is synchronous
+        lappend modules::(synchronous) $namespace
+        set module $modules::($namespace,module)
         if {[lsearch -exact $names $module]<0} {
             lappend names $module
         }
@@ -292,7 +297,7 @@ proc refresh {} {
 set draggable [expr {!$readOnly}]
 set x 0.0
 set y 0.0
-foreach module $modules::(indexed) {
+foreach module $modules::(namespaces) {
     if {[info exists ${module}::data(views)]} {                                                        ;# user overrode default view
         set viewMembers [set ${module}::data(views)]                                      ;# create and manage a table for each view
     } else {
@@ -307,13 +312,19 @@ foreach module $modules::(indexed) {
         } else {                                                                                     ;# use single default data view
             set table [new dataTable $global::canvas -data ${module}::data -draggable $draggable]
         }
+        if {[info exists modules::($module,identifier)]} {                                             ;# set a title for data table
+            set title $modules::($module,identifier)                                                ;# favor identifier if it exists
+        } else {
+            set title $module                                                                         ;# else simply use module name
+        }
         if {[info exists initializer]&&([lsearch -exact $modules::(initialized) $module]>=0)} {  ;# setup initialized modules tables
             eval composite::configure $table [record::tableOptions $initializer $module $index]
             # use stored window manager initialization data for table
             foreach {x y width height level} [record::tableWindowManagerData $initializer $module $index] {}
-            manageViewer $table 0 -static $global::static -setx $x -sety $y -setwidth $width -setheight $height -level $level
+            manageViewer $table 0 -static $global::static -setx $x -sety $y -setwidth $width -setheight $height -level $level\
+                -title $title
         } else {
-            manageViewer $table 0 -static $global::static -setx $x -sety $y
+            manageViewer $table 0 -static $global::static -setx $x -sety $y -title $title
         }
         set x [expr {$x+$global::xWindowManagerInitialOffset}]                            ;# offset tables to achieve a nicer layout
         set y [expr {$y+$global::yWindowManagerInitialOffset}]
