@@ -1,12 +1,12 @@
 # copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: mounts.tcl,v 1.6 1999/02/26 21:13:24 jfontain Exp $}
+set rcsId {$Id: mounts.tcl,v 1.8 1999/07/23 21:26:40 jfontain Exp $}
 
 
-package provide mounts 1.3
+package provide mounts 1.4
 package require filesystem
-package require network 1.2
+package require network 1.3
 
 namespace eval mounts {
 
@@ -45,7 +45,7 @@ Filesystem types are (from the fstab manual page):
 
 Module options:
 
--r (--remote) [user@]host: remote monitoring using "user" as logname on remote host "host" (rsh facilities must be properly setup). If user is not specified, current user is used as logname on remote host.
+-r (--remote) [user@]host: remote monitoring using "user" as logname on remote host "host" (rsh facilities must be properly setup). If user is not specified, current user is used as logname on remote host. The module title is set to "mounts(host)".
 
 Examples:
 
@@ -63,6 +63,8 @@ $ moodss mounts --remote foo.bar.com
         if {![catch {set locator $options(--remote)}]||![catch {set locator $options(-r)}]} {                   ;# remote monitoring
             set data(pollTimes) {30 10 20 60 120 300 600}                                ;# poll less often when remotely monitoring
             foreach {remote(user) remote(host)} [network::parseRemoteLocator $locator] {}
+            network::checkRemoteOutputEmptiness $remote(host)
+            set data(identifier) mounts($remote(host))
             set file [open "| /usr/bin/rsh -nl $remote(user) $remote(host) cat /proc/version"]
             fileevent $file readable {set ::mounts::remote(busy) 0}
             vwait ::mounts::remote(busy)                                                               ;# do not hang user interface
