@@ -1,14 +1,15 @@
 # copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: ps.tcl,v 1.34 1999/02/26 21:26:57 jfontain Exp $}
+set rcsId {$Id: ps.tcl,v 1.38 1999/07/24 10:59:11 jfontain Exp $}
 
 
 # System dependent process data access procedures. Only Linux is supported at this time (who needs another OS?).
 # Corresponding pkgIndex.tcl file content: package ifneeded ps 1.4 "source [file join $dir ps.tcl]"
+proc echo {args} {foreach argument $args {puts -nonewline "$argument "}; puts {}}
 
-package provide ps 1.6
-package require network 1.2
+package provide ps 1.7
+package require network 1.3
 
 namespace eval ps {
 
@@ -19,7 +20,7 @@ namespace eval ps {
         2,label %CPU 2,type real 2,message {processor usage in percent}
         3,label %memory 3,type real 3,message {memory usage in percent}
         4,label RSS 4,type integer 4,message {real memory size in kiloBytes}
-        5,label TTY 5,type dictionary 5,message {terminal number}
+        5,label TTY 5,type dictionary 5,message {terminal device (may be empty)}
         6,label status 6,type ascii 6,message {state: Running, Sleeping (D: uninterruptible), Zombie, Traced (or stopped)}
         7,label name 7,type dictionary 7,message {filename of the executable} 7,anchor left
         8,label {command line} 8,type dictionary 8,message {full command line} 8,anchor left
@@ -29,7 +30,7 @@ In the first table, tasks running on the system are initially sorted, with the m
 
 Module options:
 
--r (--remote) [user@]host: remote monitoring using "user" as logname on remote host "host" (rsh facilities must be properly setup). If user is not specified, current user is used as logname on remote host.
+-r (--remote) [user@]host: remote monitoring using "user" as logname on remote host "host" (rsh facilities must be properly setup). If user is not specified, current user is used as logname on remote host. The module title is set to "ps(host)".
 
 Examples:
 
@@ -52,12 +53,24 @@ $ moodss ps --remote foo.bar.com
             set data(pollTimes) {30 20 60 120 300 600}                                   ;# poll less often when remotely monitoring
             set data(2,message) {processor usage in percent (approximated)}                                         ;# see help text
             foreach {remote(user) remote(host)} [network::parseRemoteLocator $locator] {}
+            network::checkRemoteOutputEmptiness $remote(host)
+            set data(identifier) ps($remote(host))
         } else {
             set data(pollTimes) {20 10 30 60 120 300 600}
         }
         updateUserNameArray                                               ;# inititialize user identifier to user name mapping array
     }
 
+    proc ldelete {listName value} {
+        upvar $listName list
+
+        set index [lsearch -exact $list $value]
+        if {$index<0} {
+            error "\"$value\" is not in list"
+        }
+        set list [lreplace $list $index $index]
+    }
+
     proc updateUserNameArray {} {
         variable remote
         variable userName
@@ -76,22 +89,43 @@ $ moodss ps --remote foo.bar.com
         close $file
     }
 
-    array set pseudoTty {128 P 144 Q 160 R 176 S 192 p 208 q 224 r 240 s}
-
-    proc ttyName {number} {                                                        ;# convert a terminal number into a terminal name
+    proc ttyName {number} {                     ;# convert a terminal number into a terminal name (see linux/Documentation/proc.txt)
         variable pseudoTty
 
-        if {($number>>8)!=4} {
-            return {}
-        }
+        set major [expr {$number>>8}]
         set minor [expr {$number&0xFF}]
-        if {$minor<64} {
-            return $minor
-        } elseif {$minor<128} {
-            return S[expr {$minor&0x3F}]
-        } else {
-            return $pseudoTty([expr {$minor&0xF0}])[format %x [expr {$minor&0x0F}]]
+        switch $major {
+            2 {
+                return pty$minor
+            }
+            3 {
+                return ttyp$minor
+            }
+            4 {
+                if {$minor<64} {
+                    return tty$minor
+                } elseif {$minor<68} {
+                    return ttyS$minor
+                }
+            }
+            5 {
+                switch $minor {
+                    0 {return tty}
+                    1 {return console}
+                    2 {return ptmx}
+                }
+                if {($minor>=64)&&($minor<68)} {
+                    return cua$minor
+                }
+            }
+            128 {
+                return ptm$minor
+            }
+            136 {
+                return pts$minor
+            }
         }
+        return {}                                                                                                  ;# not a terminal
     }
 
     proc nullToSpace {data} {                       ;# replace null characters with space characters ### use regsub with Tcl 8.1 ###
@@ -178,7 +212,6 @@ $ moodss ps --remote foo.bar.com
 
             updateProcessData $pid $uptime $clock $memoryTotal $line
         }
-
         cd $currentDirectory
         cleanupProcessesData current                                                          ;# clean up disappeared processes data
         incr data(updates)
