@@ -1,7 +1,7 @@
 # copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: record.tcl,v 1.21 1999/07/14 15:05:10 jfontain Exp $}
+set rcsId {$Id: record.tcl,v 1.25 1999/07/18 15:59:20 jfontain Exp $}
 
 
 class record {
@@ -37,7 +37,7 @@ class record {
         puts $file "width [winfo width .] height [winfo height .]"
         puts $file "pollTime $global::pollTime"
         puts $file "modules \{"
-        foreach module $modules::(indexed) {
+        foreach module $modules::(namespaces) {
             puts $file "    $module \{"
             puts $file "        arguments \{$modules::($module,arguments)\}"
             puts $file "        tables \{"
@@ -86,11 +86,11 @@ class record {
         close $file
     }
 
-    proc modules {this} {
+    proc modules {this} {                            ;# indexed modules must be sorted so that they are created in the correct order
         variable ${this}data
 
         array set data [set ${this}data(modules)]
-        return [array names data]
+        return [lsort -dictionary [array names data]]
     }
 
     proc modulesWithArguments {this} {           ;# return list of format: module [-option [value] -option ...] module [-option ...]
@@ -165,4 +165,5 @@ class record {
         }
         return $options
     }
+
 }
