@@ -1,7 +1,7 @@
 # copyright (C) 1997-1999 Jean-Luc Fontaine (mailto:jfontain@multimania.com)
 # this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu
 
-set rcsId {$Id: viewer.tcl,v 1.29 1999/07/14 15:48:04 jfontain Exp $}
+set rcsId {$Id: viewer.tcl,v 1.31 1999/07/18 14:01:56 jfontain Exp $}
 
 class viewer {                                                                             ;# handle viewers related functionalities
     # viewers do not derive from a common interface class but rather support a common set a options through the composite class
@@ -70,14 +70,14 @@ class viewer {
     }
 
     proc label {array row column} {   ;# derive a label from cell using the tabular data index columns. array name must be qualified
+        set label {}
+        set identifier [modules::identifier $array]                                             ;# see if array needs identification
+        if {[string length $identifier]>0} {                     ;# data comes from a module array (could come from a summary table)
+            set label "$identifier: "
+        }
         if {[catch {set ${array}(indexColumns)} columns]} {                                                      ;# no index columns
             set columns 0                                                                 ;# use first column as single index column
         }
-        if {$modules::(duplicates)} {                                                     ;# there are several instances of a module
-            set label "[modules::identifier $array] "                                          ;# so module identification is needed
-        } else {
-            set label {}
-        }
         foreach index $columns {                                                                      ;# use index columns for label
             if {[catch {set ${array}($row,$index)} value]} {
                 append label {? }               ;# cell may no longer exist if originating from a save file, give user some feedback
